@@ -2,16 +2,20 @@
 # this repository contains the full copyright notices and license terms.
 import random
 import time
+from collections import defaultdict
 from email.header import Header
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from email.utils import formataddr, getaddresses
 from functools import lru_cache
 from urllib.parse import (
-    parse_qs, parse_qsl, urlencode, urlsplit, urlunsplit, urljoin)
+    parse_qs, parse_qsl, urlencode, urljoin, urlsplit, urlunsplit)
 
+from genshi.core import END, START, Attrs, QName
 from genshi.template import MarkupTemplate
-from genshi.core import START, END, QName, Attrs
+from sql import Literal
+from sql.aggregate import Count
+
 try:
     import html2text
 except ImportError:
@@ -21,18 +25,20 @@ from trytond.config import config
 from trytond.i18n import gettext
 from trytond.ir.session import token_hex
 from trytond.model import (
-    DeactivableMixin, Workflow, ModelSQL, ModelView, Unique, fields)
+    DeactivableMixin, Index, ModelSQL, ModelView, Unique, Workflow, fields)
 from trytond.pool import Pool
 from trytond.pyson import Eval
-from trytond.report import Report
-from trytond.report import get_email
-from trytond.sendmail import sendmail_transactional, SMTPDataManager
-from trytond.tools import grouped_slice
-from trytond.transaction import Transaction
+from trytond.report import Report, get_email
+from trytond.sendmail import SMTPDataManager, sendmail_transactional
+from trytond.tools import grouped_slice, reduce_ids
+from trytond.tools.email_ import (
+    EmailNotValidError, convert_ascii_email, normalize_email, set_from_header,
+    validate_email)
+from trytond.transaction import Transaction, inactive_records
 from trytond.url import http_host
-from trytond.wizard import Wizard, StateView, StateTransition, Button
+from trytond.wizard import Button, StateTransition, StateView, Wizard
 
-from .exceptions import TemplateError
+from .exceptions import EMailValidationError, TemplateError
 
 if not config.get(
         'html', 'plugins-marketing.email.message-content'):
@@ -47,7 +53,7 @@ URL_OPEN = urljoin(URL_BASE, '/m/empty.gif')
 def _formataddr(name, email):
     if name:
         name = str(Header(name, 'utf-8'))
-    return formataddr((name, email))
+    return formataddr((name, convert_ascii_email(email)))
 
 
 def _add_params(url, **params):
@@ -86,14 +92,10 @@ class Email(DeactivableMixin, ModelSQL, ModelView):
             ('email_list_unique', Unique(t, t.email, t.list_),
                 'marketing_email.msg_email_list_unique'),
             ]
-
-    @classmethod
-    def __register__(cls, module_name):
-        super().__register__(module_name)
-
-        table_h = cls.__table_handler__(module_name)
-        table_h.index_action(['email', 'list_'], action='add')
-        table_h.index_action(['list_', 'active'], action='add')
+        cls._sql_indexes.add(Index(
+                t,
+                (t.list_, Index.Equality()),
+                (t.email, Index.Equality())))
 
     @classmethod
     def default_email_token(cls, nbytes=None):
@@ -128,7 +130,7 @@ class Email(DeactivableMixin, ModelSQL, ModelView):
     @classmethod
     def _format_email(cls, records):
         for record in records:
-            email = record.email.lower()
+            email = normalize_email(record.email).lower()
             if email != record.email:
                 record.email = email
         cls.save(records)
@@ -150,6 +152,26 @@ class Email(DeactivableMixin, ModelSQL, ModelView):
         records = sum(args[0:None:2], [])
         cls._format_email(records)
 
+    @classmethod
+    def validate_fields(cls, records, fields_names):
+        super().validate_fields(records, fields_names)
+        cls.check_valid_email(records, fields_names)
+
+    @classmethod
+    def check_valid_email(cls, records, fields_names=None):
+        if fields_names and 'email' not in fields_names:
+            return
+        for record in records:
+            if record.email:
+                try:
+                    validate_email(record.email)
+                except EmailNotValidError as e:
+                    raise EMailValidationError(gettext(
+                            'marketing_email.msg_email_invalid',
+                            record=record.rec_name,
+                            email=record.email),
+                        str(e)) from e
+
     def get_email_subscribe(self, report_name='marketing.email.subscribe'):
         pool = Pool()
         ActionReport = pool.get('ir.action.report')
@@ -175,7 +197,7 @@ class Email(DeactivableMixin, ModelSQL, ModelView):
         # Make it slow to prevent brute force attacks
         delay = config.getint('marketing', 'subscribe_delay', default=1)
         Transaction().atexit(time.sleep, delay)
-        with Transaction().set_context(active_test=False):
+        with inactive_records():
             records = cls.search([
                     ('email_token', 'in', tokens),
                     ])
@@ -241,6 +263,8 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
     name = fields.Char("Name", required=True)
     language = fields.Many2One('ir.lang', "Language", required=True)
     emails = fields.One2Many('marketing.email', 'list_', "Emails")
+    subscribed = fields.Function(
+        fields.Integer("Subscribed"), 'get_subscribed')
 
     @staticmethod
     def default_language():
@@ -256,6 +280,24 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
         except ValueError:
             return None
 
+    @classmethod
+    def get_subscribed(cls, lists, name):
+        pool = Pool()
+        Email = pool.get('marketing.email')
+        email = Email.__table__()
+        cursor = Transaction().connection.cursor()
+
+        subscribed = defaultdict(int)
+        query = email.select(
+            email.list_, Count(Literal('*')), group_by=[email.list_])
+        for sub_lists in grouped_slice(lists):
+            query.where = (
+                reduce_ids(email.list_, sub_lists)
+                & email.active)
+            cursor.execute(*query)
+            subscribed.update(cursor)
+        return subscribed
+
     def request_subscribe(self, email, from_=None):
         pool = Pool()
         Email = pool.get('marketing.email')
@@ -265,7 +307,7 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
         Transaction().atexit(time.sleep, random.random())
 
         email = email.lower()
-        with Transaction().set_context(active_test=False):
+        with inactive_records():
             records = Email.search([
                     ('email', '=', email),
                     ('list_', '=', self.id),
@@ -279,10 +321,11 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
             from_cfg = (config.get('marketing', 'email_from')
                 or config.get('email', 'from'))
             msg, title = record.get_email_subscribe()
-            msg['From'] = from_ or from_cfg
+            set_from_header(msg, from_cfg, from_ or from_cfg)
             msg['To'] = record.email
             msg['Subject'] = Header(title, 'utf-8')
-            sendmail_transactional(from_cfg, [record.email], msg)
+            sendmail_transactional(
+                from_cfg, [convert_ascii_email(record.email)], msg)
 
     def request_unsubscribe(self, email, from_=None):
         pool = Pool()
@@ -293,7 +336,7 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
         Transaction().atexit(time.sleep, random.random())
 
         email = email.lower()
-        with Transaction().set_context(active_test=False):
+        with inactive_records():
             records = Email.search([
                     ('email', '=', email),
                     ('list_', '=', self.id),
@@ -304,10 +347,11 @@ class EmailList(DeactivableMixin, ModelSQL, ModelView):
                 from_cfg = (config.get('marketing', 'email_from')
                     or config.get('email', 'from'))
                 msg, title = record.get_email_unsubscribe()
-                msg['From'] = from_ or from_cfg
+                set_from_header(msg, from_cfg, from_ or from_cfg)
                 msg['To'] = record.email
                 msg['Subject'] = Header(title, 'utf-8')
-                sendmail_transactional(from_cfg, [record.email], msg)
+                sendmail_transactional(
+                    from_cfg, [convert_ascii_email(record.email)], msg)
 
 
 class Message(Workflow, ModelSQL, ModelView):
@@ -318,34 +362,37 @@ class Message(Workflow, ModelSQL, ModelView):
     _states = {
         'readonly': Eval('state') != 'draft',
         }
-    _depends = ['state']
     from_ = fields.Char(
-        "From", states=_states, depends=_depends,
+        "From", states=_states,
         help="Leave empty for the value defined in the configuration file.")
     list_ = fields.Many2One(
         'marketing.email.list', "List",
-        required=True, states=_states, depends=_depends)
+        required=True, states=_states)
     title = fields.Char(
-        "Title", required=True, states=_states, depends=_depends)
+        "Title", required=True, states=_states)
     content = fields.Text(
         "Content",
         states={
             'required': Eval('state') != 'draft',
             'readonly': _states['readonly'],
-            },
-        depends=['state'] + _depends)
+            })
     urls = fields.One2Many(
         'web.shortened_url', 'record', "URLs", readonly=True)
     state = fields.Selection([
             ('draft', "Draft"),
             ('sending', "Sending"),
             ('sent', "Sent"),
-            ], "State", readonly=True, select=True)
-    del _states, _depends
+            ], "State", readonly=True, sort=False)
+    del _states
 
     @classmethod
     def __setup__(cls):
         super().__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(
+                t, (t.state, Index.Equality()),
+                where=t.state.in_(['draft', 'sending'])))
         cls._transitions |= set([
                 ('draft', 'sending'),
                 ('sending', 'sent'),
@@ -371,22 +418,25 @@ class Message(Workflow, ModelSQL, ModelView):
         return 'draft'
 
     @classmethod
-    def validate(cls, messages):
-        super().validate(messages)
-        for message in messages:
-            message.check_content()
+    def validate_fields(cls, messages, field_names):
+        super().validate_fields(messages, field_names)
+        cls.check_content(messages, field_names)
 
-    def check_content(self):
-        if not self.content:
+    @classmethod
+    def check_content(cls, messages, field_names=None):
+        if field_names and 'content' not in field_names:
             return
-        try:
-            MarkupTemplate(self.content)
-        except Exception as exception:
-            raise TemplateError(
-                gettext('marketing_email'
-                    '.msg_message_invalid_content',
-                    message=self.rec_name,
-                    exception=exception)) from exception
+        for message in messages:
+            if not message.content:
+                continue
+            try:
+                MarkupTemplate(message.content)
+            except Exception as exception:
+                raise TemplateError(
+                    gettext('marketing_email'
+                        '.msg_message_invalid_content',
+                        message=message.rec_name,
+                        exception=exception)) from exception
 
     @classmethod
     @ModelView.button
@@ -468,7 +518,7 @@ class Message(Workflow, ModelSQL, ModelView):
                 to = _formataddr(name, email.email)
 
                 msg = MIMEMultipart('alternative')
-                msg['From'] = message.from_ or from_cfg
+                set_from_header(msg, from_cfg, message.from_ or from_cfg)
                 msg['To'] = to
                 msg['Subject'] = Header(message.title, 'utf-8')
                 if html2text:
@@ -526,5 +576,4 @@ class SendTestView(ModelView):
         'marketing.email', "E-Mail", required=True,
         domain=[
             ('list_', '=', Eval('list_')),
-            ],
-        depends=['list_'])
+            ])
