@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from werkzeug.wrappers import Response
 
+from trytond.protocols.wrappers import Response
 from trytond.tools import file_open
 from trytond.wsgi import app
 
